@@ -20,8 +20,6 @@ namespace stan {
      * @param stan_lang_in Stan model specification
      * @param cpp_out C++ code output stream
      * @param model_name Name of model class
-     * @param in_file_name Name of input file to use in error
-     * messages; defaults to <code>input</code>.
      *
      * @return <code>false</code> if code could not be generated
      *    due to syntax error in the Stan model;
@@ -30,10 +28,9 @@ namespace stan {
     bool compile(std::ostream* msgs,
                  std::istream& stan_lang_in,
                  std::ostream& cpp_out,
-                 const std::string& model_name,
-                 const std::string& in_file_name = "input") {
+                 const std::string& model_name) {
       program prog;
-      bool parsed_ok = parse(msgs, stan_lang_in, in_file_name,
+      bool parsed_ok = parse(msgs, stan_lang_in,
                              model_name, prog);
       if (!parsed_ok)
         return false;  // syntax error in program
