@@ -1,24 +1,20 @@
 #ifndef STAN_MCMC_HMC_HAMILTONIANS_DIAG_E_METRIC_HPP
 #define STAN_MCMC_HMC_HAMILTONIANS_DIAG_E_METRIC_HPP
 
+#include <stan/mcmc/hmc/hamiltonians/base_hamiltonian.hpp>
+#include <stan/mcmc/hmc/hamiltonians/diag_e_point.hpp>
 #include <boost/random/variate_generator.hpp>
 #include <boost/random/normal_distribution.hpp>
 
-#include <stan/mcmc/hmc/hamiltonians/base_hamiltonian.hpp>
-#include <stan/mcmc/hmc/hamiltonians/diag_e_point.hpp>
-
 namespace stan {
-
   namespace mcmc {
 
     // Euclidean manifold with diagonal metric
-    template <typename Model, typename BaseRNG>
+    template <class Model, class BaseRNG>
     class diag_e_metric: public base_hamiltonian<Model, diag_e_point, BaseRNG> {
     public:
-      diag_e_metric(Model& model, std::ostream* e)
-        : base_hamiltonian<Model, diag_e_point, BaseRNG>(model, e) {}
-
-      ~diag_e_metric() {}
+      explicit diag_e_metric(const Model& model)
+        : base_hamiltonian<Model, diag_e_point, BaseRNG>(model) {}
 
       double T(diag_e_point& z) {
         return 0.5 * z.p.dot( z.mInv.cwiseProduct(z.p) );
@@ -32,15 +28,27 @@ namespace stan {
         return this->V(z);
       }
 
-      const Eigen::VectorXd dtau_dq(diag_e_point& z) {
+      double dG_dt(diag_e_point& z,
+                   interface_callbacks::writer::base_writer& info_writer,
+                   interface_callbacks::writer::base_writer& error_writer) {
+        return 2 * T(z) - z.q.dot(z.g);
+      }
+
+      Eigen::VectorXd dtau_dq(
+        diag_e_point& z,
+        interface_callbacks::writer::base_writer& info_writer,
+        interface_callbacks::writer::base_writer& error_writer) {
         return Eigen::VectorXd::Zero(this->model_.num_params_r());
       }
 
-      const Eigen::VectorXd dtau_dp(diag_e_point& z) {
+      Eigen::VectorXd dtau_dp(diag_e_point& z) {
         return z.mInv.cwiseProduct(z.p);
       }
 
-      const Eigen::VectorXd dphi_dq(diag_e_point& z) {
+      Eigen::VectorXd dphi_dq(
+        diag_e_point& z,
+        interface_callbacks::writer::base_writer& info_writer,
+        interface_callbacks::writer::base_writer& error_writer) {
         return z.g;
       }
 
@@ -54,7 +62,5 @@ namespace stan {
     };
 
   }  // mcmc
-
 }  // stan
-
 #endif
