@@ -1,6 +1,7 @@
 #ifndef STAN_MODEL_UTIL_HPP
 #define STAN_MODEL_UTIL_HPP
 
+#include <stan/interface_callbacks/writer/base_writer.hpp>
 #include <stan/math/fwd/scal/fun/square.hpp>
 #include <stan/math/fwd/core.hpp>
 #include <stan/math/prim/mat/fun/Eigen.hpp>
@@ -17,10 +18,10 @@
 #include <stan/math/fwd/mat/functor/jacobian.hpp>
 #include <stan/math/rev/mat/functor/jacobian.hpp>
 #include <stan/math/mix/mat/functor/partial_derivative.hpp>
-
 #include <cmath>
 #include <iomanip>
 #include <iostream>
+#include <string>
 #include <vector>
 
 namespace stan {
@@ -269,8 +270,7 @@ namespace stan {
      * @param params_i Integer-valued parameter vector.
      * @param epsilon Real-valued scalar saying how much to perturb. Reasonable value is 1e-6.
      * @param error Real-valued scalar saying how much error to allow. Reasonable value is 1e-6.
-     * @param o Output stream for messages. 
-     * @param msgs Stream to which Stan programs write. 
+     * @param writer Writer for messages
      * @return number of failed gradient comparisons versus allowed
      * error, so 0 if all gradients pass
      */
@@ -280,15 +280,17 @@ namespace stan {
                        std::vector<int>& params_i,
                        double epsilon,
                        double error,
-                       std::ostream& o,
-                       std::ostream* msgs) {
+                       stan::interface_callbacks::writer::base_writer& writer) {
+      std::stringstream msg;
       std::vector<double> grad;
       double lp
         = log_prob_grad<propto, jacobian_adjust_transform>(model,
                                                            params_r,
                                                            params_i,
                                                            grad,
-                                                           msgs);
+                                                           &msg);
+      if (msg.str().length() > 0)
+        writer(msg.str());
 
       std::vector<double> grad_fd;
       finite_diff_grad<false,
@@ -296,28 +298,36 @@ namespace stan {
                        M>(model,
                           params_r, params_i,
                           grad_fd, epsilon,
-                          msgs);
+                          &msg);
+      if (msg.str().length() > 0)
+        writer(msg.str());
 
       int num_failed = 0;
 
-      o << std::endl
-        << " Log probability=" << lp
-        << std::endl;
+      msg.str("");
+      msg << " Log probability=" << lp;
+
+      writer();
+      writer(msg.str());
+      writer();
+
+      msg.str("");
+      msg << std::setw(10) << "param idx"
+          << std::setw(16) << "value"
+          << std::setw(16) << "model"
+          << std::setw(16) << "finite diff"
+          << std::setw(16) << "error";
+
+      writer(msg.str());
 
-      o << std::endl
-        << std::setw(10) << "param idx"
-        << std::setw(16) << "value"
-        << std::setw(16) << "model"
-        << std::setw(16) << "finite diff"
-        << std::setw(16) << "error"
-        << std::endl;
       for (size_t k = 0; k < params_r.size(); k++) {
-        o << std::setw(10) << k
-          << std::setw(16) << params_r[k]
-          << std::setw(16) << grad[k]
-          << std::setw(16) << grad_fd[k]
-          << std::setw(16) << (grad[k] - grad_fd[k])
-          << std::endl;
+        msg.str("");
+        msg << std::setw(10) << k
+            << std::setw(16) << params_r[k]
+            << std::setw(16) << grad[k]
+            << std::setw(16) << grad_fd[k]
+            << std::setw(16) << (grad[k] - grad_fd[k]);
+        writer(msg.str());
         if (std::fabs(grad[k] - grad_fd[k]) > error)
           num_failed++;
       }
@@ -417,6 +427,22 @@ namespace stan {
       stan::math::gradient(model_functional<M>(model, msgs), x, f, grad_f);
     }
 
+    template <class M>
+    void gradient(const M& model,
+                  const Eigen::Matrix<double, Eigen::Dynamic, 1>& x,
+                  double& f,
+                  Eigen::Matrix<double, Eigen::Dynamic, 1>& grad_f,
+                  stan::interface_callbacks::writer::base_writer& writer) {
+      std::stringstream ss;
+      stan::math::gradient(model_functional<M>(model, &ss), x, f, grad_f);
+      // FIXME(DL): remove blank line at the end of the gradient call
+      //            this assumes the last character is a newline
+      //            this matches the v2.8.0 behavior
+      std::string msg = ss.str();
+      if (msg.length() > 1)
+        writer(msg.substr(0, msg.length() - 1));
+    }
+
     template <class M>
     void hessian(const M& model,
                  const Eigen::Matrix<double, Eigen::Dynamic, 1>& x,
