@@ -1,6 +1,7 @@
 #ifndef STAN_VARIATIONAL_NORMAL_MEANFIELD_HPP
 #define STAN_VARIATIONAL_NORMAL_MEANFIELD_HPP
 
+#include <stan/interface_callbacks/writer/base_writer.hpp>
 #include <stan/math/prim/mat/fun/Eigen.hpp>
 #include <stan/math/prim/scal/fun/constants.hpp>
 #include <stan/math/prim/mat/meta/get.hpp>
@@ -367,8 +368,7 @@ namespace stan {
        * @param[in] n_monte_carlo_grad Number of samples for gradient
        * computation.
        * @param[in,out] rng Random number generator.
-       * @param[in,out] print_stream Stream for convergence
-       * assessment output.
+       * @param[in,out] message_writer writer for messages
        * @throw std::domain_error If the number of divergent
        * iterations exceeds its specified bounds.
        */
@@ -378,7 +378,8 @@ namespace stan {
                      Eigen::VectorXd& cont_params,
                      int n_monte_carlo_grad,
                      BaseRNG& rng,
-                     std::ostream* print_stream) const {
+                     interface_callbacks::writer::base_writer& message_writer)
+        const {
         static const char* function =
           "stan::variational::normal_meanfield::calc_grad";
 
@@ -404,7 +405,10 @@ namespace stan {
             eta(d) = stan::math::normal_rng(0, 1, rng);
           zeta = transform(eta);
           try {
-            stan::model::gradient(m, zeta, tmp_lp, tmp_mu_grad, print_stream);
+            std::stringstream ss;
+            stan::model::gradient(m, zeta, tmp_lp, tmp_mu_grad, &ss);
+            if (ss.str().length() > 0)
+              message_writer(ss.str());
             stan::math::check_finite(function, "Gradient of mu", tmp_mu_grad);
             mu_grad += tmp_mu_grad;
             omega_grad.array() += tmp_mu_grad.array().cwiseProduct(eta.array());
