@@ -1,7 +1,8 @@
 #ifndef STAN_MATH_PRIM_SCAL_PROB_PARETO_TYPE_2_CDF_HPP
 #define STAN_MATH_PRIM_SCAL_PROB_PARETO_TYPE_2_CDF_HPP
 
-#include <boost/random/variate_generator.hpp>
+#include <stan/math/prim/scal/meta/is_constant_struct.hpp>
+#include <stan/math/prim/scal/meta/partials_return_type.hpp>
 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
 #include <stan/math/prim/scal/err/check_finite.hpp>
@@ -15,6 +16,7 @@
 #include <stan/math/prim/scal/meta/contains_nonconstant_struct.hpp>
 #include <stan/math/prim/scal/meta/VectorView.hpp>
 #include <stan/math/prim/scal/meta/VectorBuilder.hpp>
+#include <boost/random/variate_generator.hpp>
 #include <cmath>
 
 namespace stan {
@@ -139,7 +141,7 @@ namespace stan {
           operands_and_partials.d_x4[n] *= P;
       }
 
-      return operands_and_partials.to_var(P, y, mu, lambda, alpha);
+      return operands_and_partials.value(P);
     }
   }
 }
