@@ -1,21 +1,18 @@
 #ifndef STAN_MATH_REV_ARR_FUNCTOR_COUPLED_ODE_SYSTEM_HPP
 #define STAN_MATH_REV_ARR_FUNCTOR_COUPLED_ODE_SYSTEM_HPP
 
-#include <stan/math/prim/mat/meta/get.hpp>
 #include <stan/math/prim/arr/meta/get.hpp>
-#include <stan/math/prim/mat/meta/length.hpp>
 #include <stan/math/prim/arr/meta/length.hpp>
-
+#include <stan/math/prim/scal/err/check_equal.hpp>
+#include <stan/math/prim/arr/functor/coupled_ode_system.hpp>
 #include <stan/math/rev/scal/fun/value_of_rec.hpp>
 #include <stan/math/rev/scal/fun/value_of.hpp>
 #include <stan/math/rev/core.hpp>
-#include <stan/math/prim/scal/err/check_equal.hpp>
-#include <stan/math/prim/arr/functor/coupled_ode_system.hpp>
 #include <ostream>
+#include <stdexcept>
 #include <vector>
 
 namespace stan {
-
   namespace math {
 
     // This code is in this directory because it includes var
@@ -214,29 +211,23 @@ namespace stan {
       std::vector<std::vector<stan::math::var> >
       decouple_states(const std::vector<std::vector<double> >& y) {
         using stan::math::precomputed_gradients;
-        std::vector<stan::math::var> temp_vars;
-        std::vector<double> temp_gradients;
+        std::vector<stan::math::var> temp_vars(N_);
+        std::vector<double> temp_gradients(M_);
         std::vector<std::vector<stan::math::var> > y_return(y.size());
 
         for (size_t i = 0; i < y.size(); i++) {
-          temp_vars.clear();
-
           // iterate over number of equations
           for (size_t j = 0; j < N_; j++) {
-            temp_gradients.clear();
-
             // iterate over parameters for each equation
             for (size_t k = 0; k < M_; k++)
-              temp_gradients.push_back(y[i][y0_dbl_.size()
-                                            + y0_dbl_.size() * k + j]);
+              temp_gradients[k] = y[i][y0_dbl_.size() + y0_dbl_.size() * k + j];
 
-            temp_vars.push_back(precomputed_gradients(y[i][j],
-                                                      theta_,
-                                                      temp_gradients));
+            temp_vars[j] = precomputed_gradients(y[i][j],
+                                                 theta_,
+                                                 temp_gradients);
           }
           y_return[i] = temp_vars;
         }
-
         return y_return;
       }
     };
@@ -421,25 +412,20 @@ namespace stan {
         using stan::math::var;
         using std::vector;
 
-        vector<var> temp_vars;
-        vector<double> temp_gradients;
+        vector<var> temp_vars(N_);
+        vector<double> temp_gradients(N_);
         vector<vector<var> > y_return(y.size());
 
         for (size_t i = 0; i < y.size(); i++) {
-          temp_vars.clear();
-
           // iterate over number of equations
           for (size_t j = 0; j < N_; j++) {
-            temp_gradients.clear();
-
             // iterate over parameters for each equation
             for (size_t k = 0; k < N_; k++)
-              temp_gradients.push_back(y[i][y0_.size() + y0_.size() * k + j]);
+              temp_gradients[k] = y[i][y0_.size() + y0_.size() * k + j];
 
-            temp_vars.push_back(precomputed_gradients(y[i][j],
-                                                      y0_, temp_gradients));
+            temp_vars[j] = precomputed_gradients(y[i][j],
+                                                 y0_, temp_gradients);
           }
-
           y_return[i] = temp_vars;
         }
 
@@ -646,23 +632,19 @@ namespace stan {
         vector<var> vars = y0_;
         vars.insert(vars.end(), theta_.begin(), theta_.end());
 
-        vector<var> temp_vars;
-        vector<double> temp_gradients;
+        vector<var> temp_vars(N_);
+        vector<double> temp_gradients(N_ + M_);
         vector<vector<var> > y_return(y.size());
 
         for (size_t i = 0; i < y.size(); i++) {
-          temp_vars.clear();
-
           // iterate over number of equations
           for (size_t j = 0; j < N_; j++) {
-            temp_gradients.clear();
-
             // iterate over parameters for each equation
             for (size_t k = 0; k < N_ + M_; k++)
-              temp_gradients.push_back(y[i][N_ + N_ * k + j]);
+              temp_gradients[k] = y[i][N_ + N_ * k + j];
 
-            temp_vars.push_back(precomputed_gradients(y[i][j],
-                                                      vars, temp_gradients));
+            temp_vars[j] = precomputed_gradients(y[i][j],
+                                                 vars, temp_gradients);
           }
           y_return[i] = temp_vars;
         }
@@ -670,10 +652,7 @@ namespace stan {
         return y_return;
       }
     };
-
-
-  }
-
-}
+  }  // math
+}  // stan
 
 #endif
