@@ -3,5 +3,7 @@
 
 #include <stan/math/rev/mat/functor/gradient.hpp>
 #include <stan/math/rev/mat/functor/jacobian.hpp>
+#include <stan/math/rev/mat/functor/ode_system.hpp>
+#include <stan/math/rev/mat/functor/integrate_ode_bdf.hpp>
 
 #endif
